library(ape)
library(boot)
library(car)
library(cowplot)
library(data.table)
library(doBy)
library(e1071)
library(effects)
library(geometry)
library(geomorph)
library(ggdendro)
library(ggplot2)
library(grid)
library(gridExtra)
library(ICC)
library(lattice)
library(lme4)
library(lmerTest)
library(lmodel2)
library(MASS)
#library(MethComp)
library(mice)
library(MuMIn)
library(mvShapiroTest)
library(plyr)
library(pwr)
library(R2WinBUGS)
library(RColorBrewer)
library(reshape)
library(reshape2)
library(rgl)
library(shapes)
library(shiny)
library(sjstats)
library(sparcl)
library(vegan)


# weight watchers points
ww <- function(kcal, fat=0) ceiling(2*(0.11*fat + 0.0165*kcal))/2 # rounded up to halfs 
wv <- function(kcal, fat=0) round(0.11*fat + 0.0165*kcal, 2)



### function for calculating the mode
### from https://www.tutorialspoint.com/r/r_mean_median_mode.htm
getmode <- function(v) {
  uniqv <- unique(v)
  uniqv[which.max(tabulate(match(v, uniqv)))]
}












##########################################
#####  Replace NAs of measurements of body part from one side
#####  with measurements from the other side (e.g., left and right wing)
#####  Function created with the generous help from Seraina Klopfstein
##########################################

# df dataframe
# g grouping variable, like a code number, identifying each individual

na.repl <- function(df, g){
  for(i in levels(g)){
    for (j in 1:2){
      na.elements <- which(is.na(df[which(g == i),])[j,])
      if (length(na.elements) > 0){
        for (k in na.elements){
          if (j == 1) {
            m <- 2
          } else m <- 1
          df[which(g == i)[j], k] <- df[which(g == i)[m], k]
        }
      }
    }
  }  
  list(na.repl=df)
}











##########################################
#####  One way MANOVA permutation test, mainly after 
#####  https://stat.ethz.ch/pipermail/r-help/2007-March/127749.html
##########################################

manova.perm <- function(y, g, nperm=999, teststat="Pillai") {
  
  y <- as.matrix(y)  # dependent variables
  g <- factor(g)  # grouping variable
  nperm <- nperm  # number of permutations
  teststat  # test statistic used in manova, should be either "Pillai" (default for function 'manova()', most robust), "Wilks" (most popular), "Hotelling–Lawley", or "Roy"
  
  # allocation of storage for the observed statistic
  res <- numeric(nperm+1)
  
  # do the loop - the "seq(along " res[-1])" part apparently should avoid certain problems; I cannot see for the moment, which ones!
  for(i in seq(along = res[-1])) {
    res[i] <- summary(manova(y ~ sample(g)), test = teststat)$stats[1,2]
  }
  
  # appending the observed statistic at the end of the result vector 'res'
  # also storing this in 'obs' for convenience
  res[nperm+1] <- obs <- summary(manova(y ~ g), test = teststat)$stats[1,2]
  
  # calculation of permutation p-value - the proportion of the nperm
  # permutations + 1 that are  greater than or equal to the 
  # observed statistic 'obs'
  if(teststat=="Wilks") { 
    p_value <- sum(res <= obs) / (nperm+1)
  } else {
    p_value <- sum(res >= obs) / (nperm+1)
  }
  list(p_value=p_value, resperm=res, statistic=obs)
}













##########################################
#####  pooled within group regression 
#####  à la Klingenberg
##########################################

# x independent variable(s)
# y dependent variable(s)
# g grouping variable contains 1:nlevels(g)
# xp
# yp

pooled.wg.reg <- function(x, y, g) {
	x <- x
	y <- log(y)
	g <- as.numeric(g)
	x.meang <- aggregate(x, list(g), FUN=mean); x.meang <- as.matrix(x.meang[,-1])
	y.meang <- aggregate(y, list(g), FUN=mean); y.meang <- as.matrix(y.meang[,-1])
	xp <- as.matrix(x-x.meang[g,])
	yp <- as.matrix(y-y.meang[g,])
	y.mean <- apply(y, 2, mean)
	lm <- lm(yp~xp)
	b <- t(as.matrix(lm$coefficients[2,]));
	dx <- as.matrix(x-mean(x))
	dy <- t(t(y)-as.numeric(y.mean))
	pred <- dx%*%b
	predm <- t(t(pred)+y.mean)  # predict plus mean
	resid <- dy-pred
}










##########################################
#####  calculating major axis for multiple groups 
##########################################

pc1 <- function(x, g, rounding=2) {  
x <- data.frame(x)  # data frame of measurements
g <- as.factor(g)  # grouping variable, e.g. specimen number
n <- length(g)/nlevels(g)  # sample size of each group
variable <- colnames(x) # names of variables
k <- 0
m.error <- matrix(NA, nrow=ncol(x), ncol=1); m.error <- as.vector(m.error) # transforming matrix A to vector as placeholder for measurement errors

	for (i in 1:ncol(x)) {
		k <- k+1
		model <- aov(x[,i] ~ g)
		anova <- anova(model)
		msa <- anova[1,3]  # among groups mean sums of squares (= among groups variance)
		mse <- anova[2,3]  # within groups mean sums of squares (= within groups variance)
		sa <- 1/n*(msa-mse)  # added variance component among groups (Sokal & Rohlf 1995: 197)
		merror <- mse/(mse+sa)*100  
		m.error[k] <- round(merror, rounding)
	}
	me <- data.frame(variable, m.error)
	re <- data.frame(variable, reliability=(100-m.error))
	list(me=me, re=re)
}












##########################################
#####  function for calculating reliability R and its confidence interval limits
#####  based on the Intraclass Correlation Coefficient
#####  of the package ICC (Wolak et al. 2012) for all variables of a data frame 
##########################################

icc.R <- function(g, x, rounding=2) {  
  x <- data.frame(x)  # data frame qith measurements
  g <- factor(g)  # repeatedly measured items (e.g., specimens)
  d <- dim(x)[2]  # number of numeric variables
  varname <- colnames(x) # names of variables
  k <- 0
  A <- matrix(NA, nrow=d, ncol=3) # matrix serving as placeholder
  for (i in 1:d) {
    k <- k+1
    icc <- ICCest(g, x[,i]); icc <- round(icc$ICC*100, rounding) # reliability coefficient
    lci <- ICCest(g, x[,i]); lci <- round(lci$LowerCI*100, rounding) # lower confidence interval limit of reliability coefficient
    uci <- ICCest(g, x[,i]); uci <- round(uci$UpperCI*100, rounding) # upper confidence interval limit of reliability coefficient
    A[k,] <- cbind(icc, lci, uci) 
  }
  colnames(A) <- c("R", "LCI", "UCI")
  A <- as.data.frame(A); A <- data.frame(varname, A)
  list(Rtable = A, R = A[,2], LCI = A[,3], UCI = A[,4])
}










##########################################
#####  function for calculating measurement error using Model II ANOVA, 
#####  after Bailey & Byrnes (1990) and Sokal & Rohlf (1995: 196-197, 208-214). 
#####  correct only for equal sample sizes! 
#####  in principal deprecated by more universal icc.R(), kept only for reasons of nostalgia!
##########################################

m.error <- function(g, x, rounding=2) {  
  x <- data.frame(x)  # data frame of measurements
  g <- as.factor(g)  # grouping variable, e.g. specimen number
  n <- length(g)/nlevels(g)  # sample size of each group
  variable <- colnames(x) # names of variables
  k <- 0
  m.error <- matrix(NA, nrow=ncol(x), ncol=1); m.error <- as.vector(m.error) # transforming matrix A to vector as placeholder for measurement errors
  
  for (i in 1:ncol(x)) {
    k <- k+1
    model <- aov(x[,i] ~ g)
    anova <- anova(model)
    msa <- anova[1,3]  # among groups mean sums of squares (= among groups variance)
    mse <- anova[2,3]  # within groups mean sums of squares (= within groups variance)
    sa <- 1/n*(msa-mse)  # added variance component among groups (Sokal & Rohlf 1995: 197)
    merror <- mse/(mse+sa)*100  
    m.error[k] <- round(merror, rounding)
  }
  me <- data.frame(variable, m.error)
  re <- data.frame(variable, reliability=(100-m.error))
  list(me=me, re=re)
}










###################################################
##### Gold standard Bland-Altman method for single (Bland & Altman (1986 [2010], 
##### 1999 and Carstensen et al. (2008)) and repeated measurments
##### (using the package MethComp of Carstensen et al. (2008))
###################################################

# meth			group variable, methods or test persons
# item	     	specimen variable
# y				vector of numeric variable
# gs			person or method considered as the gold standard
# comp			person or method to be compared with gold standard

bland.altman <- function(meth, item, y, gs, comp, ylim=limits, pch=18, colp=4, colb=6, cex=1.3, ...){ 
	data <- data.frame(meth, item=as.factor(item), y); data <- data[order(data$meth, 	data$item),]
	
	if(length(data[,1])/nlevels(data$item)>2){
	data1 <- subset(data, meth==gs)
	data2 <- subset(data, meth==comp)
	d <- data1$y-data2$y # difference of measurments
	y.mean <- (data1$y+data2$y)/2 # mean of measurments
	limits <- c(-(max(y.mean)-min(y.mean))/2, (max(y.mean)-min(y.mean))/2)
	bias<- mean(d) # mean difference of measurments
	
	# calculate Limits of Agreement
	datameth <- Meth(data, print=FALSE)
	ba.est <- BA.est(datameth, linked=FALSE)
	levels.meth <- levels(datameth$meth)
	levels.gs <- c(gs, comp)
	
	# keep up with the sequence of methods/persons as specified with gs and comp
	if(levels.meth[1]==levels.gs[1]){
	LoA <- ba.est$LoA[c(2,3)]*-1
	} else {		
	LoA <- ba.est$LoA[c(2,3)]	
	}
	# plot(y.mean, d, pch = pch, col=colp, cex = 1.3, ylim = ylim, xlab=paste("(", gs, " [= gold standard] + ", comp, ") / 2", sep=""), ylab=paste(gs, " [= gold standard] - ", comp, sep=""), ...)
	plot(y.mean, d, pch = pch, col=colp, cex = cex, ylim = ylim, xlab=paste("(", gs, " + ", comp, ") / 2", sep=""), ylab=paste(gs, " - ", comp, sep=""), ...)
	abline(h=c(bias, LoA[1], LoA[2]), lwd=c(1.5,1.2,1.2), lty=c(1,2,2), col=c(colb,"grey25","grey25"))
	list(bias=round(bias,2), LoA=round(LoA, 3))
	
	
	} else {
	data1 <- subset(data, meth==gs); data1 <- data1[order(data1$item),]
	data2 <- subset(data, meth==comp); data2 <- data2[order(data2$item),]
	d <- data1$y-data2$y # difference of measurments
	y.mean <- (data1$y+data2$y)/2 # mean of measurments
	limits <- c(-(max(y.mean)-min(y.mean))/2, (max(y.mean)-min(y.mean))/2)
	# y.gs <- data1$y # mean of measurments
	# limits <- c(-(max(y.gs)-min(y.gs))/2, (max(y.gs)-min(y.gs))/2)
	bias<- mean(d) # mean difference of measurments
	sd.d <- sd(d) # standard deviation of difference
	n <- length(d)
	# sd.value <- 2  # value suggested by Bland & Altman (2010: 933)
	sd.value <- qt(0.975, df = n-1)*sqrt(1+1/n)  # value suggested by Carstensen et al. (2008)
	LoA = c(bias-sd.value*sd.d, bias+sd.value*sd.d)
	se.bias <- sd.d/sqrt(n)
	se.LoA <- 1.71*se.bias
	l.cf.bias <- bias - qt(0.975, df = n-1) * se.bias # lower 95% confidence limit of mean
	u.cf.bias <- bias + qt(0.975, df = n-1) * se.bias # upper 95% confidence limit of mean
	cf.l.LoA <- c(LoA[1] - (qt(0.975, df = n-1) * se.LoA), LoA[1] + (qt(0.975, df = n-1) * se.LoA)) # 95% confidence limits of lower limit of agreement
	cf.u.LoA <- c(LoA[2] - (qt(0.975, df = n-1) * se.LoA), LoA[2] + (qt(0.975, df = n-1) * se.LoA)) # 95% confidence limits of upper limit of agreement
	# plot(y.mean, d, pch = pch, col=colp, cex = 1.3, ylim = ylim, xlab=paste("(", gs, " [= gold standard] + ", comp, ") / 2", sep=""), ylab=paste(gs, " [= gold standard] - ", comp, sep=""), ...)
	plot(y.mean, d, pch = pch, col=colp, cex = 1.3, ylim = ylim, xlab=paste("(", gs, " + ", comp, ") / 2", sep=""), ylab=paste(gs, " - ", comp, sep=""), ...)
	# text(y.mean, d, labels=item[1:nlevels(item)], pos=3, offset=0.2, srt=0, cex=0.8, ...)
	abline(h = bias, lty = 1, lwd = 1.5, col = colb)
	abline(h = LoA[1], lty = 3, lwd = 2, col = "grey50")
	abline(h = LoA[2], lty = 3, lwd = 2, col = "grey50")
	# abline(h = u.cf.bias, lty = 2, lwd = 0.7, col = "grey50")
	# abline(h = l.cf.bias, lty = 2, lwd = 0.7, col = "grey50")
	list(bias = round(bias,2), LoA=round(LoA, 3), s.bias = round(sd.d,2), se.bias=round(se.bias, 2), se.LoA = round(se.LoA,2), cf95.bias=round(c(l.cf.bias, u.cf.bias), 3), cf95.low.LoA = round(cf.l.LoA,2), cf95.upp.LoA = round(cf.u.LoA,2))
	}
	} 






###################################################
##### Gold standard Bland-Altman method for repeated measurments
##### using the package MethComp of Carstensen et al. (2008)
###################################################

# meth			group variable, methods or test persons
# item	     	specimen variable
# y				vector of numeric variable
# gs			person or method considered as the gold standard
# comp			person or method to be compared with gold standard

bland.altman.repeated <- function(meth, item, y, gs, comp, ylim=limits, pch=18, col=4, ...) {
	data <- data.frame(meth, item=as.factor(item), y); data <- data[order(data$meth, 	data$item),]
	data1 <- subset(data, meth==gs)
	data2 <- subset(data, meth==comp)
	d <- data1$y-data2$y # difference of measurments
	y.mean <- (data1$y+data2$y)/2 # mean of measurments
	limits <- c(-(max(y.mean)-min(y.mean))/2, (max(y.mean)-min(y.mean))/2)
	bias<- mean(d) # mean difference of measurments
	
	# calculate Limits of Agreement
	datameth <- Meth(data, print=FALSE)
	ba.est <- BA.est(datameth, linked=FALSE)
	levels.meth <- levels(datameth$meth)
	levels.gs <- c(gs, comp)
	
	# keep up with the sequence of methods/persons as specified with gs and comp
	if(levels.meth[1]==levels.gs[1]){
	LoA <- ba.est$LoA[c(2,3)]*-1
	} else {		
	LoA <- ba.est$LoA[c(2,3)]	
	}
	plot(y.mean, d, pch = pch, cex = 1.3, ylim = ylim, xlab=paste("(", gs, " [= gold standard] + ", comp, ") / 2", sep=""), ylab=paste(gs, " [= gold standard] - ", comp, sep=""), col=col, ...)
	abline(h=c(bias, LoA[1], LoA[2]), lwd=c(2,2,2), lty=c(1,3,3), col=c(3,"grey50","grey50"))
	list(bias=round(bias,2), LoA=round(LoA, 3))
	}









###################################################
##### Gold standard Bland-Altman method for single measurments after
##### Bland & Altman (1986 [2010], 1999) and Carstensen et al. (2008)
###################################################

# meth			group variable, methods or test persons
# item	     	specimen variable
# y				vector of numeric variable
# gs			person or method considered as the gold standard
# comp			person or method to be compared with gold standard

bland.altman.single <- function(meth, item, y, gs, comp, ylim=limits, pch=18, ...) {
	data <- data.frame(meth, item, y)
	data1 <- subset(data, meth==gs); data1 <- data1[order(data1$item),]
	data2 <- subset(data, meth==comp); data2 <- data2[order(data2$item),]
	d <- data1$y-data2$y # difference of measurments
	y.mean <- (data1$y+data2$y)/2 # mean of measurments
	limits <- c(-(max(y.mean)-min(y.mean))/2, (max(y.mean)-min(y.mean))/2)
	# y.gs <- data1$y # mean of measurments
	# limits <- c(-(max(y.gs)-min(y.gs))/2, (max(y.gs)-min(y.gs))/2)
	bias<- mean(d) # mean difference of measurments
	sd.d <- sd(d) # standard deviation of difference
	n <- length(d)
	# sd.value <- 2  # value suggested by Bland & Altman (2010: 933)
	sd.value <- qt(0.975, df = n-1)*sqrt(1+1/n)  # value suggested by Carstensen et al. (2008)
	LoA = c(bias-sd.value*sd.d, bias+sd.value*sd.d)
	se.bias <- sd.d/sqrt(n)
	se.LoA <- 1.71*se.bias
	l.cf.bias <- bias - qt(0.975, df = n-1) * se.bias # lower 95% confidence limit of mean
	u.cf.bias <- bias + qt(0.975, df = n-1) * se.bias # upper 95% confidence limit of mean
	cf.l.LoA <- c(LoA[1] - (qt(0.975, df = n-1) * se.LoA), LoA[1] + (qt(0.975, df = n-1) * se.LoA)) # 95% confidence limits of lower limit of agreement
	cf.u.LoA <- c(LoA[2] - (qt(0.975, df = n-1) * se.LoA), LoA[2] + (qt(0.975, df = n-1) * se.LoA)) # 95% confidence limits of upper limit of agreement
	plot(y.mean, d, pch = pch, cex = 1.3, ylim = ylim, xlab=paste("(", gs, " [= gold standard] + ", comp, ") / 2", sep=""), ylab=paste(gs, " [= gold standard] - ", comp, sep=""), ...)
	# text(y.mean, d, labels=item[1:nlevels(item)], pos=3, offset=0.2, srt=0, cex=0.8, ...)
	abline(h = bias, lty = 1, lwd = 1.5, col = 6)
	abline(h = LoA[1], lty = 3, lwd = 2, col = "grey50")
	abline(h = LoA[2], lty = 3, lwd = 2, col = "grey50")
	abline(h = u.cf.bias, lty = 2, lwd = 0.7, col = "grey50")
	abline(h = l.cf.bias, lty = 2, lwd = 0.7, col = "grey50")
	list(bias = round(bias,2), LoA=round(LoA, 3), s.bias = round(sd.d,2), se.bias=round(se.bias, 2), se.LoA = round(se.LoA,2), cf95.bias=round(c(l.cf.bias, u.cf.bias), 3), cf95.low.LoA = round(cf.l.LoA,2), cf95.upp.LoA = round(cf.u.LoA,2))
	}














###################################################
##### generate random sample
###################################################

# X     a dataframe of numeric variables
# n     number of randomly selected specimens

random.sample <- function(X, n) {
	return(X[sample(nrow(X), n), ])	
}




###################################################
##### generate random sample of a dataset partitioned in three equal parts
###################################################

# DF    a full dataframe including factorial variables
# X     a dataframe of numeric variables
# n     number of randomly selected specimens
# d     nominator for dividing the dataset


part.random.sample <- function(DF, X, n, d) {
	geosize <- exp(apply(log(X), 1, mean)) # calculating geometric mean size
	DF <- cbind(DF, geosize) # add geosize to dataframe
	DF <- DF[order(DF$geosize, decreasing=FALSE),] # sort dataframe by geosize
	dim <- dim(chal)[1]
	dim3 <- floor(dim(chal)[1]/d)
	DF.1 <- DF[1:dim3,]
	DF.2 <- DF[(dim3+1):(dim-dim3),]
	DF.3 <- DF[(dim-dim3+1):(dim),]
	DF1 <- random.sample(DF.1, n)
	DF2 <- random.sample(DF.2, n)
	DF3 <- random.sample(DF.3, n)

	return(rbind(DF1, DF2, DF3))
}








###################################################
##### generate random sample of a dataset partitioned in upper and lower third
###################################################

# DF    a full dataframe including factorial variables
# X     a dataframe of numeric variables
# nr    number of randomly selected specimens


twopart.random.sample <- function(DF, X, n) {
	geosize <- exp(apply(log(X), 1, mean)) # calculating geometric mean size
	DF <- cbind(DF, geosize) # add geosize to dataframe
	DF <- DF[order(DF$geosize, decreasing=FALSE),] # sort dataframe by geosize
	dim <- dim(chal)[1]
	dim3 <- floor(dim(chal)[1]/3)
	DF.1 <- DF[1:dim3,]
	DF.3 <- DF[(dim-dim3+1):(dim),]
	DF1 <- random.sample(DF.1, n)
	DF3 <- random.sample(DF.3, n)

	return(rbind(DF1, DF3))
}








##########################################
##### extracts numeric data from a data frame 
##### !! Assumes numeric variables = data variables in a data frame (df)
##########################################
num.data <- function(x) {
	nums <- sapply(x, is.numeric); u <- chal[,nums]  # select raw coordinates from df
	u
}






##########################################
##### Transforms m x n matrix of raw coordinates to p x k x n array. 
##### !! Assumes numeric variables = data variables in a data frame (df) and 
##### !! columns sorted as x01,y01, x02,y02, ...
##########################################
matrix.array <- function(x) {
	nums <- sapply(x, is.numeric); y <- x[,nums]  # select raw coordinates from df
	y.a <- y[,seq(1, ncol(y), by=2)]  # sort df by x coordinates
	y.b <- y[,seq(2, ncol(y), by=2)]  # sort df by y coordinates
	y.ab <- cbind(y.a, y.b) # unite y.a and y.b to a single df
	#### transform raw coordinates to p x k x n array
	u <- as.matrix(y.ab); u <- t(u) # transform df to matrix and transpose matrix
	dim(u) <- c(ncol(y.ab)/2, 2, length(y.ab[,1])) # determine dimensions of array
	u
}







	
###################################################
##### calculating all k(k-1)/2 distances of a landmark configuration
###################################################

##### Arguments
 # x           a numeric matrix (or data frame) raw coordinates sorted as x01,y01, x02,y02, ...
 # v           number of variables to be removed from the data frame D of distances between landmarks
 
##### Values
 # D           a data frame with distances between landmarks

lmdist <- function(u, v=0) {
	x <- u[,seq(1, ncol(u), by=2)]
	y <- u[,seq(2, ncol(u), by=2)]
	k <- 0
	l <- 0
	m <- 0
	X <- matrix(NA,nrow=nrow(x), ncol=ncol(x)*(ncol(x)-1)/2) 
	for (i in 1:(ncol(x)-1)) for(j in (i+1):ncol(x)){
	k <- k+1
	X[,k] <- (x[,i]-x[,j])^2
	}
	Y <- matrix(NA,nrow=nrow(y), ncol=ncol(y)*(ncol(y)-1)/2) 
	for (i in 1:(ncol(y)-1)) for(j in (i+1):ncol(y)){
	l <- l+1
	Y[,l] <- (y[,i]-y[,j])^2
	}
	p <- colnames(X, do.NULL=F, prefix="lm")
	distnames <- matrix(NA,nrow=1, ncol(x)*(ncol(x)-1)/2) 
		for (i in 1:(ncol(x)-1)) for(j in (i+1):ncol(x)){
		m <- m+1
		pi <- p[i]
		pj <- p[j]
		pij <- paste(pi, pj, sep=".")
		distnames[m] <- pij
	}
		
	X <- data.frame(X)
	Y <- data.frame(Y)
	D <- sqrt(X+Y)
	colnames(D) <- distnames
	Dm <- apply(D, 2, mean)
	D <- rbind(Dm, D)
	D <- D[order(D[1,], decreasing=TRUE)]
	v <- v
	D <- D[2:dim(D)[1], 1:(dim(D)[2]-v)]
	D
}










###################################################
##### calculating all k(k-1)/2 distances of a landmark configuration
##### and selecting distances according to level of measurement error
###################################################

##### Arguments
 # x           a numeric matrix (or data frame) raw coordinates sorted as x01,y01, x02,y02, ...
 # sp         factor variable used for calculating measurement error
 # e          maximum level of measurement error for selecting distances 
  
##### Values
 # D           a data frame with distasnces between landmarks

lmdistrel <- function(g, u, reliability=0.95) {
	x <- u[,seq(1, ncol(u), by=2)]
	y <- u[,seq(2, ncol(u), by=2)]
	k <- 0
	l <- 0
	m <- 0
	X <- matrix(NA,nrow=nrow(x), ncol=ncol(x)*(ncol(x)-1)/2) 
	for (i in 1:(ncol(x)-1)) for(j in (i+1):ncol(x)){
	k <- k+1
	X[,k] <- (x[,i]-x[,j])^2
	}
	Y <- matrix(NA,nrow=nrow(y), ncol=ncol(y)*(ncol(y)-1)/2) 
	for (i in 1:(ncol(y)-1)) for(j in (i+1):ncol(y)){
	l <- l+1
	Y[,l] <- (y[,i]-y[,j])^2
	}
	p <- colnames(X, do.NULL=F, prefix="lm")
	distnames <- matrix(NA,nrow=1, ncol(x)*(ncol(x)-1)/2) 
		for (i in 1:(ncol(x)-1)) for(j in (i+1):ncol(x)){
		m <- m+1
		pi <- p[i]
		pj <- p[j]
		pij <- paste(pi, pj, sep=".")
		distnames[m] <- pij
	}
	X <- data.frame(X)
	Y <- data.frame(Y)
	D <- sqrt(X+Y)
	colnames(D) <- distnames
	ICC <- icc.rel(g, D)
	icc <- ICC$reliability
	icc <- icc$reliability
	D <- rbind(icc, D)
	reliability <- reliability
	D <- subset(D, select = D[1,] > reliability)
	D <- D[-1,]
	D
}










##########################################
##### standard error, Crawley 2007, p. 54 
##########################################
se <- function(x) sqrt(var(x)/length(x))





	
##########################################
##### confidence intervalls, Crawley 2007, p. 55 
##########################################
se <- function(x) sqrt(var(x)/length(x))
ci95 <- function(x) {
	t.value <- qt(0.975, length(x)-1)
	standard.error <- se(x)
	ci <- t.value*standard.error
	cat("95% Confidence Interval = ", mean(x) -ci, "to ", mean(x) +ci) }
	





##########################################
##### Thorpe standardization after Bartels et al., 2011, J Zool Syst Evol Res, 49: 17-25. This is equivalent to the correction method of Janzon (1986, Syst Ent 11: 75-82)
##########################################

Thorpe <- function(x,y) {
	X0 <- log(mean(x, na.rm=TRUE))
	X <- log(x)
	Y <- log(y)
	reg <- lm(Y~X) # linear regression
	b <- reg$coefficients[2] # slope of linear regression
	Yc <- Y-b*(X-X0) # Thorpe standardization
	Yc <- exp(Yc)
	}








##########################################
##### Abline Major Axis
##########################################

ma.abline <- function(x,y,...) {
	lm <- lmodel2(y~x)
	ma.intercept <- lm$regression.results[[2]][2]
	ma.slope <- lm$regression.results[[3]][2]
	# abline(ma.intercept, ma.slope, ...)
	list(slope=ma.slope, intercept=ma.intercept)
}





##########################################
##### scatterplot with ggplot2
##########################################


ggscatter <- function(xvar, yvar, shape, color, shapes=shapes, colors=colors, xlab=xlab, ylab=ylab, col.lab=col.lab, shape.lab=shape.lab, symb.size=symb.size, xlab.size=16, ylab.size=16) {
	df <- data.frame(xvar, yvar, shape, color)
	plot <- ggplot(df, aes(xvar, yvar, shape = shape, color = color))
	plot <- plot + geom_point(size=symb.size) + theme(aspect.ratio=1)
	plot <- plot + scale_shape_manual(values=shapes) + scale_colour_manual(values=colors)
	plot <- plot  + labs(shape = shape.lab, colour = col.lab)
	plot <- plot  + xlab(xlab) + ylab(ylab) + labs(colour = "grouping", shape = "grouping")
	plot <- plot + theme(axis.title.x = element_text(size=xlab.size), axis.title.y = element_text(size=ylab.size))
	}













##########################################
##### scatterhullplot with ggplot2
##########################################


ggscatterhull <- function(xvar, yvar, shape, color, group, shapes=shapes, colors=colors, xlab=xlab, ylab=ylab, title=NULL, col.lab=col.lab, shape.lab=shape.lab) {
	df <- data.frame(xvar, yvar, shape, color, group)
	find_hull <- function(df)df[chull(df$xvar, df$yvar),]
	hulls <- ddply(df, "group", find_hull)
	plot <- plot + geom_polygon(data = hulls, aes(group=group), colour="gray75", fill = NA)
	plot <- ggplot(df, aes(xvar, yvar, shape = shape, color = color))
	plot <- plot + geom_point(size=3) + theme(aspect.ratio=1) + labs(title=title)
	plot <- plot + scale_shape_manual(values=shapes)
	plot <- plot + scale_colour_manual(values=colors)
	plot <- plot  + xlab(xlab) + ylab(ylab)
	plot + labs(shape = shape.lab, colour = col.lab)
	}
















##########################################
##### scatterhull (colored) original version
##########################################

### gives a bivariate scatterplot with convex hulls around specified groups; it's by the way my very first, really useful R-function!


# # scatterhull <- function(xvar, yvar, factor, col=as.numeric(factor), lwd=1.5, pch=as.numeric(factor)+6, cex=0.9, xlab=deparse(substitute(xvar)), ylab=deparse(substitute(yvar)), main=NA, placelegend="bottomright", legend=levels(factor), pchlegend=1:nlevels(factor)+6, collegend=1:nlevels(factor), cexlegend=1, border=0,...){
	# plot(xvar, yvar, xlab=xlab, ylab=ylab, main=main, cex.main=0.9, col=col, pch=pch, cex=cex, ...)
	# dataset <- data.frame(xvar, yvar, factor)
	# for (i in 1:nlevels(factor)){
	# dataset.1 <- dataset[as.numeric(factor)==i,]
	# hull <- chull(dataset.1$xvar, dataset.1$yvar)
	# polygon(dataset.1$xvar[hull], dataset.1$yvar[hull], border=i+border, lwd=lwd)
	# }
	# legend(placelegend, bty="n", legend, pch=pchlegend, col=collegend, cex=cexlegend)
	# }







##########################################
##### scatterhull (colored), revised version 2014.12.19
##########################################

### gives a bivariate scatterplot with convex hulls around specified groups; it's by the way my very first, really useful R-function!


scatterhull <- function(xvar, yvar, factor, col=as.numeric(factor), lwd=1.5, pch=as.numeric(factor), cex=0.9, xlab=deparse(substitute(xvar)), ylab=deparse(substitute(yvar)), main=NA, placelegend="bottomright", legend=levels(factor), pchlegend=1:nlevels(factor), collegend=1:nlevels(factor), cexlegend=1, border=0,...){
	plot(xvar, yvar, xlab=xlab, ylab=ylab, main=main, cex.main=0.9, col=col, pch=pch, cex=cex, ...)
	dataset <- data.frame(xvar, yvar, factor)
	for (i in 1:nlevels(factor)){
	dataset.1 <- dataset[as.numeric(factor)==i,]
	hull <- chull(dataset.1$xvar, dataset.1$yvar)
	polygon(dataset.1$xvar[hull], dataset.1$yvar[hull], border=i+border, lwd=lwd)
	}
	legend(placelegend, bty="n", legend, pch=pchlegend, col=collegend, cex=cexlegend)
	}







##########################################
##### scutterhull (monochrome)
##########################################

### gives a bivariate scatterplot with convex hulls around specified groups; it's by the way my very first, really useful function!


hull <- function(xvar, yvar, factor, border="lightgray", lwd=1.5, xlab=deparse(substitute(xvar)), ylab=deparse(substitute(yvar)), ...){
	plot(xvar, yvar, xlab=xlab, ylab=ylab, type="n", ...)
	dataset <- data.frame(xvar, yvar, factor)
	for (i in 1:nlevels(factor)){
	dataset.1 <- dataset[as.numeric(factor)==i,]
	hull <- chull(dataset.1$xvar, dataset.1$yvar)
	polygon(dataset.1$xvar[hull], dataset.1$yvar[hull], border=border, lwd=lwd)
	}
	}




##########################################
##### scutterhull (colored 2)
##########################################
### gives a bivariate scatterplot with convex hulls around specified groups; it's by the way my very first, really useful function!


scatterhull3 <- function(xvar, yvar, factor){
	dataset <- data.frame(xvar, yvar, factor)
	for (i in 1:nlevels(factor)){
	dataset.1 <- dataset[as.numeric(factor)==i,]
	hull <- chull(dataset.1$xvar, dataset.1$yvar)
	polygon(dataset.1$xvar[hull], dataset.1$yvar[hull], border=i)
	}
	}		












###################################################
## Determines standard distance according to Riedwyl & Ambühl 2000: 134
###################################################

standdist.mv <- function(x, f) {
	X <- x
	U <- split(X, f)
	U1 <- data.frame(U[1])
	n1 <- length(U1[,1])
	U2 <- data.frame(U[2])
	n2 <- length(U2[,1])
	lm <- lm(f~as.matrix(X))
	lms <- summary(lm)
	R2 <- lms$r.squared; R2
	stand.dist <- sqrt(((n1+n2)*(n1+n2-2)/(n1*n2))*(R2/(1-R2)))
	stand.dist
}














##########################################
##### Jolicoeur size and shape PCA on log-transformed data
##########################################

##### Arguments
 # x           a numeric matrix (or data frame) that provides the data for the principal component analysis
 # npc         number of shape components shown in numerical output, as usually only the first two or three components contain useful information. Default value is npc=3
 # rpc         number of decimal places used for printing components and loadings. Default value is rpc=4 

##### Values
 # joliPCA       returns the results of the shape PCA as an object of class "prcomp". Mainly used for displaying the screeplot
 # loadings   a matrix of variable loadings (i.e., a matrix whose columns contain the coefficients of the eigenvectors)
 # PCmatrix     a matrix of rotated data (i.e., the centered data multiplied by the loadings)
 # components     a matrix containing the variance, proportion of varaiance and cumulative proportion of variance

joliPCA <- function(x, npc=3, rpc=4, scale=FALSE, log=1){
if(log>0){
	X <- log(x)
	} else{X <- x}
X <- scale(X, center=TRUE, scale=FALSE)
PCA <- prcomp(X)
loadings <- PCA$rotation
if(sum(loadings[,1]) < 0){
	sign <- -1
	} else{sign <- 1}
loadings <- loadings*sign
allosize <- PCA$x[,1]*sign
p <- dim(PCA$x)[2]
PCmatrix <- PCA$x[,2:p]*sign		
eigenvalues.a <- PCA$sd[2:p]^2; eigenvalues.b <- eigenvalues.a/sum(eigenvalues.a); eigenvalues.c <- cumsum(eigenvalues.b); eigenvalues <- rbind(eigenvalues.a, eigenvalues.b, eigenvalues.c)
colnames(loadings) <- paste("PC", 1:p, sep="")
colnames(PCA$x) <- colnames(loadings)
colnames(eigenvalues) <- colnames(loadings)[c(1:(p-1))]; rownames(eigenvalues) <- c("Variance", "Proportion of Variance", "Cumulative Proportion")
list(PCA=PCA, allosize=allosize, PCmatrix=PCmatrix, loadings=round(loadings[,1:npc], rpc), components=round(eigenvalues[,1:npc], rpc))
}









##########################################
##### Allometry corrected PCA, Claude (2008)
##########################################


alloPCA <- function(x, factor){

Y <- x
grouping <- split(Y, factor)
group1 <- data.frame(grouping[1])
group2 <- data.frame(grouping[2])

pca <- prcomp(group1)
proj <- as.matrix(Y)%*%pca[[2]]
plot(proj[,2:3], pch=16, col=factor)

pca <- prcomp(group2)
proj <- as.matrix(Y)%*%pca[[2]]
plot(proj[,2:3], pch=16, col=factor)

}





















##########################################
##### Confidence Intervals for Jolicoeur PC
##########################################

##### Arguments
 # x          a numeric matrix (or data frame) that provides the data for the PCA ratio spectrum
 # pc         the shape component for which a PCA ratio spectrum should be drawn. Default value is pc=1
 # bootrep    the number of bootstrap replicates on which the 68% confidence intervals are based. Default value is bootrep=500 
 # barcol     colour of horizontal bars (confidence intervals). Default value is barcol="blue" 
 # barlwd     line width of horizontal bars (confidence intervals). Default value is barlwd=2.5 
 # linecol    colour of vertical line. Default value is linecol="black" 
 # linelwd    line width of vertical line. Default value is linelwd=0.5 
 # labelsize  size of variable labels. Default value is labelsize=0.9 
 # labelfont  font of variable labels. Default value is labelfont=1 
 # nosize     size of number labels. Default value is nosize=0.7 
 # nofont     font of number labels. Default value is nofont=1
 # maina      first part of main title. Default value is maina="PCA Ratio Spectrum for PC"
 # mainb      second part of main title. Default value is mainb=pc (i.e., the number of the component, for which the PCA ratio spectrum is drawn)
 # mainc      third part of main title. Default value is mainc=""
 # suba       first part of sub title. Default value is suba="bars = 68% confidence intervals based on "
 # subb       second part of sub title. Default value is subb=bootrep (i.e., the number of bootstrap replicates)
 # subc       third part of sub title. Default value is subc=" bootstrap replicates"
     
jolipcaCI <- function (x, pc=1, sign=1, bootrep=500, barcol="red", barlwd=4, linecol="black", linelwd=0.5, labelsize=0.9, labelfont=1, nosize=0.7, nofont=1, maina="Confidence Intervals for Jolicoeur PC", mainb=pc, mainc="", suba="bars = 68% confidence intervals based on ", subb=bootrep, subc=" bootstrap replicates", main=paste(maina, mainb, mainc, sep=""), sub=paste(suba, subb, subc, sep="")) {
trait.names <- colnames(x)
Y <- log(x)
Y <- scale(Y, center=TRUE, scale=FALSE)
Sigma <- cov(Y)
p <- length(trait.names)
n <- nrow(Y)
u <- eigen(Sigma)$vectors[, pc]*sign
rep.boot=bootrep
U.boot <- matrix(0, nrow=rep.boot, ncol=p)
for (i in c(1:rep.boot)){
index.boot=sample(c(1:n), n, replace=TRUE)
Y.boot <- Y[index.boot, ]
Sigma.boot <- cov(Y.boot)
u.boot <- eigen(Sigma.boot)$vectors[, pc]*sign
if (u%*%u.boot<0){
u.boot <- -u.boot
}
U.boot[i, ] <- u.boot
}
mean.u <- apply(U.boot, 2, mean)
sd.u  <- apply(U.boot, 2, sd)
mean.minus.sd <- mean.u-sd.u
mean.plus.sd <- mean.u+sd.u
m <- min(mean.u); M <- max(mean.u)
u.sorted <- sort(mean.u, decreasing=TRUE)
index <- sort(mean.u, index.return=TRUE, decreasing=TRUE)$ix
trait.names.sorted <- trait.names[index]
mean.minus.sd <- mean.minus.sd[index]
mean.plus.sd <- mean.plus.sd[index]
plot(c(0, 0), c(m, M), xlab="", ylab="", type="n", cex=1, asp=1, main=main, sub=sub, cex.lab=1, axes=FALSE)
lines(c(0, 0), c(m, M), col=linecol, lwd=linelwd)
text(0, u.sorted[1], labels=signif(u.sorted[1], digits=2), pos=3, srt=0, col="black", font=nofont, cex=nosize)
text(0, u.sorted[p], labels=signif(u.sorted[p], digits=2), pos=1, srt=0, col="black", font=nofont, cex=nosize)
for (k in 1:p){
lines(c(mean.minus.sd[k]-u.sorted[k], mean.plus.sd[k]-u.sorted[k]), c(u.sorted[k], u.sorted[k]), col=barcol, lwd=barlwd)
if(k%%2==0){
text(max(sd.u), u.sorted[k], labels=trait.names.sorted[k], pos=4, srt=0, col="black", font=labelfont, cex=labelsize)
}
if(k%%2==1){
text(-max(sd.u), u.sorted[k], labels=trait.names.sorted[k], pos=2, srt=0, col="black", font=labelfont, cex=labelsize)
}}
}



















##########################################
##### PCA in isometry free shape space
##########################################

##### Arguments
 # x           a numeric matrix (or data frame) that provides the data for the principal component analysis
 # npc         number of shape components shown in numerical output, as usually only the first two or three components contain useful information. Default value is npc=3
 # rpc         number of decimal places used for printing components and loadings. Default value is rpc=4 

##### Values
 # shapePCA    returns the results of the shape PCA as an object of class "prcomp". Mainly used for displaying the screeplot
 # loadings    a matrix of variable loadings (i.e., a matrix whose columns contain the eigenvectors)
 # components  a matrix containing the variance, proportion of varaiance and cumulative proportion of variance
 # PCmatrix    a matrix of rotated data (i.e., the centered data multiplied by the loadings)
 # isosize     a matrix containing isometric size (i.e., the centered data multiplide by the isometric size vector)

shapePCA <- function(x, npc=3, rpc=4){
X <- log(x)
X <- scale(X, center=TRUE, scale=FALSE)
p <- dim(X)[2]
I <- diag(1,p,p)
a0 <- as.vector(rep(1,p)/p)
P <- I-p*(a0%*%t(a0))
Y <- X%*%P
colnames(Y) <- colnames(X)
isosize <- X%*%a0; colnames(isosize) <- "isosize"
PCA <- prcomp(Y,center=FALSE, scale=FALSE)
loadings <- PCA$rotation
p <- dim(loadings)[2]
PCmatrix <- PCA$x
components.a <- PCA$sdev^2; components.b <- components.a/sum(components.a); components.c <- cumsum(components.b); components <- rbind(components.a, components.b, components.c)
colnames(loadings) <- paste("shape.PC", 1:p, sep="")
colnames(PCmatrix) <- colnames(loadings)
colnames(components) <- colnames(loadings); rownames(components) <- c("Variance", "Proportion of Variance", "Cumulative Proportion")
list(PCA=PCA, PCmatrix=PCmatrix[,1:npc], isosize=as.numeric(isosize), loadings=round(loadings[,1:npc], rpc), components=round(components[,1:npc], rpc))
}








##########################################
##### Isometric size
##########################################

isosize <- function(x) {
	X <- log(x)
	X <- scale(X, center=TRUE, scale=FALSE)
	p <- dim(X)[2]
	I <- diag(1,p,p)
	a0 <- as.vector(rep(1,p)/p)
	P <- I-p*(a0%*%t(a0))
	Y <- X%*%P
	colnames(Y) <- colnames(X)
	isosize <- X%*%a0
	isosize	
}







##########################################
##### Matrix of isometry free shape values
##########################################

isofreeshapes <- function(x) {

X <- log(x)
X <- scale(X, center=TRUE, scale=FALSE)
p <- dim(X)[2]
I <- diag(1,p,p)
a0 <- as.vector(rep(1,p)/p)
P <- I-p*(a0%*%t(a0))
if.shapes <- X%*%P; colnames(if.shapes) <- colnames(X)
if.shapes
}







##########################################
##### PCA Ratio Spectrum
##########################################

##### Arguments
 # x          a numeric matrix (or data frame) that provides the data for the PCA ratio spectrum
 # pc         the shape component for which a PCA ratio spectrum should be drawn. Default value is pc=1
 # bootrep    the number of bootstrap replicates on which the 68% confidence intervals are based. Default value is bootrep=500 
 # barcol     colour of horizontal bars (confidence intervals). Default value is barcol="blue" 
 # barlwd     line width of horizontal bars (confidence intervals). Default value is barlwd=2.5 
 # linecol    colour of vertical line. Default value is linecol="black" 
 # linelwd    line width of vertical line. Default value is linelwd=0.5 
 # labelsize  size of variable labels. Default value is labelsize=0.9 
 # labelfont  font of variable labels. Default value is labelfont=1 
 # nosize     size of number labels. Default value is nosize=0.7 
 # nofont     font of number labels. Default value is nofont=1
 # maina      first part of main title. Default value is maina="PCA Ratio Spectrum for PC"
 # mainb      second part of main title. Default value is mainb=pc (i.e., the number of the component, for which the PCA ratio spectrum is drawn)
 # mainc      third part of main title. Default value is mainc=""
 # suba       first part of sub title. Default value is suba="bars = 68% confidence intervals based on "
 # subb       second part of sub title. Default value is subb=bootrep (i.e., the number of bootstrap replicates)
 # subc       third part of sub title. Default value is subc=" bootstrap replicates"
     
pcaRS <- function (x, pc=1, sequence=1, bootrep=500, barcol="#1F78B4", barlwd=4, linecol="black", linelwd=0.5, labelsize=0.9, labelfont=1, nosize=0.7, nofont=1, maina="PCA Ratio Spectrum for shape PC", mainb=pc, mainc="", suba="bars = 68% confidence intervals based on ", subb=bootrep, subc=" bootstrap replicates", main=paste(maina, mainb, mainc, sep=""), sub=paste(suba, subb, subc, sep=""), ...) {
trait.names <- colnames(x)
Y <- log(x)
Y <- scale(Y, center=TRUE, scale=FALSE)
Sigma <- cov(Y)
p <- length(trait.names)
n <- nrow(Y)
I <- diag(1, p, p)
a0 <- as.vector(rep(1,p)/p)
P <- I-a0%*%solve(t(a0)%*%a0)%*%t(a0)
Sigma.1 <- P%*%Sigma%*%P
u <- (eigen(Sigma.1)$vectors[, pc])*sequence
rep.boot=bootrep
U.boot <- matrix(0, nrow=rep.boot, ncol=p)
for (i in c(1:rep.boot)){
index.boot=sample(c(1:n), n, replace=TRUE)
Y.boot <- Y[index.boot, ]
Sigma.boot <- P%*%cov(Y.boot)%*%P
u.boot <- (eigen(Sigma.boot)$vectors[, pc])*sequence
if (u%*%u.boot<0){
u.boot <- -u.boot
}
U.boot[i, ] <- u.boot
}
mean.u <- apply(U.boot, 2, mean)
sd.u  <- apply(U.boot, 2, sd)
mean.minus.sd <- mean.u-sd.u
mean.plus.sd <- mean.u+sd.u
m <- min(mean.u); M <- max(mean.u)
u.sorted <- sort(mean.u, decreasing=TRUE)
index <- sort(mean.u, index.return=TRUE, decreasing=TRUE)$ix
trait.names.sorted <- trait.names[index]
mean.minus.sd <- mean.minus.sd[index]
mean.plus.sd <- mean.plus.sd[index]
plot(c(0, 0), c(m, M), xlab="", ylab="", type="n", cex=1, asp=1, main=main, sub=sub, cex.lab=1, axes=FALSE, ...)
lines(c(0, 0), c(m, M), col=linecol, lwd=linelwd)
text(0, u.sorted[1], labels=signif(u.sorted[1], digits=2), pos=3, srt=0, col="black", font=nofont, cex=nosize)
text(0, u.sorted[p], labels=signif(u.sorted[p], digits=2), pos=1, srt=0, col="black", font=nofont, cex=nosize)
for (k in 1:p){
lines(c(mean.minus.sd[k]-u.sorted[k], mean.plus.sd[k]-u.sorted[k]), c(u.sorted[k], u.sorted[k]), col=barcol, lwd=barlwd)
if(k%%2==0){
text(max(sd.u), u.sorted[k], labels=trait.names.sorted[k], pos=4, srt=0, col="black", font=labelfont, cex=labelsize)
}
if(k%%2==1){
text(-max(sd.u), u.sorted[k], labels=trait.names.sorted[k], pos=2, srt=0, col="black", font=labelfont, cex=labelsize)
}}
}











##########################################
##### Allometry Ratio Spectrum
##########################################

##### Arguments
 # x          a numeric matrix (or data frame) that provides the data for the allometry ratio spectrum
 # bootrep    the number of bootstrap replicates on which the 68% confidence intervals are based. Default value is bootrep=500 
 # barcol     colour of horizontal bars (confidence intervals). Default value is barcol="blue" 
 # barlwd     line width of horizontal bars (confidence intervals). Default value is barlwd=2.5 
 # linecol    colour of vertical line. Default value is linecol="black" 
 # linelwd    line width of vertical line. Default value is linelwd=0.5 
 # labelsize  size of variable labels. Default value is labelsize=0.9 
 # labelfont  font of variable labels. Default value is labelfont=1 
 # nosize     size of number labels. Default value is nosize=0.7 
 # nofont     font of number labels. Default value is nofont=1
 # maina      first part of main title. Default value is maina="Allometry Ratio Spectrum"
 # mainb      second part of main title. Default value is mainb=""
 # mainc      third part of main title. Default value is mainc=""
 # suba       first part of sub title. Default value is suba="bars = 68% confidence intervals based on "
 # subb       second part of sub title. Default value is subb=bootrep (i.e., the number of bootstrap replicates)
 # subc       third part of sub title. Default value is subc=" bootstrap replicates"
     
allometryRS <- function (x, bootrep=500, sequence=1, barcol="#33A02C", barlwd=4, linecol="black", linelwd=0.5, labelsize=0.9, labelfont=1, nosize=0.7, nofont=1, maina="Allometry Ratio Spectrum", mainb="", mainc="", suba="bars = 68% confidence intervals based on ", subb=bootrep, subc=" bootstrap replicates", main=paste(maina, mainb, mainc, sep=""), sub=paste(suba, subb, subc, sep=""), ...) {
trait.names <- colnames(x)
Y <- log(x)
Y <- scale(Y,center=TRUE,scale=FALSE)
Sigma <- cov(Y)
p <- length(trait.names)
n <- nrow(Y)
a0 <- as.vector(rep(1,p)/p)
a <- a0 
d <- as.vector(Sigma%*%a)*sequence
rep.boot=bootrep
D.boot <- matrix(0, nrow=rep.boot, ncol=p)
for (i in c(1:rep.boot)){
index.boot=sample(c(1:n), n, replace=TRUE)
Y.boot <- Y[index.boot, ]
Sigma.boot <- cov(Y.boot)
d.boot <- as.vector(Sigma.boot%*%a)*sequence
d.boot <- d.boot/sqrt(d.boot%*%d.boot)
if (d%*%d.boot<0){
d.boot <- -d.boot
}
D.boot[i, ] <- d.boot
}
mean.d <- apply(D.boot, 2, mean); mean.d
sd.d <- apply(D.boot, 2, sd)
mean.minus.sd <- mean.d-sd.d
mean.plus.sd <- mean.d+sd.d
m <- min(mean.d); M <- max(mean.d)
d.sorted <- sort(mean.d, decreasing=TRUE)
index <- sort(mean.d, index.return=TRUE, decreasing=TRUE)$ix
trait.names.sorted <- trait.names[index]
mean.minus.sd <- mean.minus.sd[index]
mean.plus.sd <- mean.plus.sd[index]
plot(c(0, 0), c(m, M), xlab=NA, ylab=NA, type="n", axes=FALSE, asp=1, main=main, sub=sub, cex=1, cex.lab=1, ...)
lines(c(0, 0), c(m, M), col=linecol, lwd=linelwd)
text(0, d.sorted[1], labels=signif(d.sorted[1], digits=2), pos=3, srt=0, col="black", font=nofont, cex=nosize)
text(0, d.sorted[p], labels=signif(d.sorted[p], digits=2), pos=1, srt=0, col="black", font=nofont, cex=nosize)
for (k in 1:p){
lines(c(mean.minus.sd[k]-d.sorted[k], mean.plus.sd[k]-d.sorted[k]), c(d.sorted[k], d.sorted[k]), col=barcol, lwd=barlwd)
if(k%%2==0){
text(max(sd.d), d.sorted[k], labels=trait.names.sorted[k], pos=4, srt=0, col="black", font=labelfont, cex=labelsize)
}
if(k%%2==1){
text((-max(sd.d)), d.sorted[k], labels=trait.names.sorted[k], pos=2, srt=0, col="black", font=labelfont, cex=labelsize)}}
}









##########################################
##### LDA Ratio Extractor for two groups
##########################################

##### Arguments
 # x         a numeric matrix (or data frame)
 # g         a grouping variable with just levels
 # kmax      maximum number of ratio to be extracted. Default value is kmax=2
 # rd        the ratio of which standard distance (D.bij), standard distance for shape values (D.shape), standard distance for size values (D.shape), and delta should be calculated. Default value is rd=1
 # roundD    rounding of digits for D.bij, D.shape, D.size and delta. Default value is roundD=3 

##### Values
 # Ratio no. a list of the best ratios for separating the two groups
 # D.bij     standard distance of two groups for a particular ratio
 # D.shape   amount of difference in shape
 # D.size    amount of difference in size
 # delta     a measure of how well size discriminates in comparision to shape, calculated as delta = D.size/(D.shape + D.size)
# rd = knnen die ratios gewhlt werden

ldaRE <- function(x, g, rd=1, kmax=4, roundD=3, placelegend="bottomleft", placelegend2="bottomleft", main1="First and second best ratio", main2="Third and fourth best ratio") {
X <- data.frame(x)
groups <- g
species.names <- levels(groups)
nb.species <- length(species.names)

trait.names <- colnames(X)
p <- length(trait.names)
X1 <- (X[groups==species.names[1],1:p]) 
X2 <- (X[groups==species.names[2],1:p])
n1 <- nrow(X1)
n2 <- nrow(X2)
Y1 <- as.matrix(log(X1))
Y2 <- as.matrix(log(X2))
m1 <- colMeans(Y1)
m2 <- colMeans(Y2)
m12 <- as.vector(m1-m2)
m12 <- m12/sqrt(sum(m12^2))
Y.pooled <- rbind(scale(Y1,center=TRUE,scale=FALSE),scale(Y2,center=TRUE,scale=FALSE))
Sigma <- cov(Y.pooled)
w <- solve(Sigma)%*%m12
standard.distance <- function(x,y){
x <- as.vector(x)
y <- as.vector(y)
D <- abs(mean(x)-mean(y))/(sd(c(scale(x,center=TRUE,scale=FALSE),c(scale(y,center=TRUE,scale=FALSE)))))
return(D)
}

D.tot <- standard.distance(Y1%*%w,Y2%*%w)
I <- diag(1,p,p)
a0 <- as.vector(rep(1,p)/p)
M.k <- matrix(a0,ncol=1)
k.max <- kmax
best.ratio <- matrix(rep(0,2*k.max),ncol=2)
for (k in c(1:k.max)){
P.k <- I-M.k%*%solve(t(M.k)%*%M.k)%*%t(M.k)
Sigma.k <- P.k%*%Sigma%*%P.k
Sigma.k <- 0.5*(Sigma.k+t(Sigma.k))
ev <- eigen(Sigma.k)$values[1:(p-k)]
V <- eigen(Sigma.k)$vectors
Lambda.plus <- diag(c(1/ev,rep(0,k))) 
Sigma.plus <- V%*%Lambda.plus%*%t(V)
w.k <- Sigma.plus%*%P.k%*%m12
w.k <- as.matrix(w.k,ncol=1)
if (k==rd){ 
  D.bij <- standard.distance(Y1%*%w.k,Y2%*%w.k)
  D.shape <- standard.distance(Y1%*%w.k,Y2%*%w.k)/D.tot
  D.size <- standard.distance(Y1%*%a0,Y2%*%a0)/D.tot
  delta <- D.size/(D.size+D.shape)
}
c.max <- 0
for (i in c(1:(p-1))){  
  for (j in c((1+i):p)){
    b.ij <- rep(0,p)
    b.ij[i] <- 1
    b.ij[j] <- -1
    b.ij <- as.matrix(b.ij,ncol=1)  
    c.ij <- (t(b.ij)%*%Sigma%*%w.k)^2/(t(b.ij)%*%Sigma%*%b.ij)
    if (c.ij>c.max){
      best.b.ij <- b.ij 
      c.max <- c.ij
      best.ratio[k,1] <- i
      best.ratio[k,2] <- j 
    }    
}}
print(paste("Ratio no. ", as.character(k), ": ", trait.names[best.ratio[k,1]], "/", trait.names[best.ratio[k,2]], sep=""))
M.k <- cbind(M.k,Sigma%*%best.b.ij)
}
ratio1 <- x[,best.ratio[1,1]]/x[,best.ratio[1,2]]
ratio2 <- x[,best.ratio[2,1]]/x[,best.ratio[2,2]]
ratio3 <- x[,best.ratio[3,1]]/x[,best.ratio[3,2]]
ratio4 <- x[,best.ratio[4,1]]/x[,best.ratio[4,2]]

### plotting the best 4 ratios
scatterhull(ratio1, ratio2, xlab=paste(trait.names[best.ratio[1,1]],"/",trait.names[best.ratio[1,2]]), ylab=paste(trait.names[best.ratio[2,1]],"/",trait.names[best.ratio[2,2]]), main=main1, g, placelegend=placelegend)
# scatterhull(ratio3, ratio4, xlab=paste(trait.names[best.ratio[3,1]],"/",trait.names[best.ratio[3,2]]), ylab=paste(trait.names[best.ratio[4,1]],"/",trait.names[best.ratio[4,2]]), main=main2, g, placelegend=placelegend2)

D.bij <- paste("D.bij of ratio no. ", as.character(rd), ": ", trait.names[best.ratio[rd,1]], " : ", trait.names[best.ratio[rd,2]], " = ", as.character(round(D.bij, roundD)), sep="")
D.shape <- paste("D.shape of ratio no. ", as.character(rd), ": ", trait.names[best.ratio[rd,1]], " : ", trait.names[best.ratio[rd,2]], " = ", as.character(round(D.shape, roundD)), sep="")
D.size <- paste("D.size of ratio no. ", as.character(rd), ": ", trait.names[best.ratio[rd,1]], " : ", trait.names[best.ratio[rd,2]], " = ", as.character(round(D.size, roundD)), sep="")
delta <- paste("delta of ratio no. ", as.character(rd), ": ", trait.names[best.ratio[rd,1]], " : ", trait.names[best.ratio[rd,2]], " = ", as.character(round(delta, roundD)), sep="")
list(ratio1=ratio1, ratio2=ratio2, ratio3=ratio3, ratio4=ratio4,  D.bij=D.bij, D.shape=D.shape, D.size=D.size, delta=delta)
}





##### LDA Ratio Extractor without printing graphs

ldaRE2 <- function(x, g, rd=1, kmax=4, roundD=3, placelegend="bottomleft", placelegend2="bottomleft", main1="First and second best ratio", main2="Third and fourth best ratio") {
X <- data.frame(x)
groups <- g
species.names <- levels(groups)
nb.species <- length(species.names)

trait.names <- colnames(X)
p <- length(trait.names)
X1 <- (X[groups==species.names[1],1:p]) 
X2 <- (X[groups==species.names[2],1:p])
n1 <- nrow(X1)
n2 <- nrow(X2)
Y1 <- as.matrix(log(X1))
Y2 <- as.matrix(log(X2))
m1 <- colMeans(Y1)
m2 <- colMeans(Y2)
m12 <- as.vector(m1-m2)
m12 <- m12/sqrt(sum(m12^2))
Y.pooled <- rbind(scale(Y1,center=TRUE,scale=FALSE),scale(Y2,center=TRUE,scale=FALSE))
Sigma <- cov(Y.pooled)
w <- solve(Sigma)%*%m12

standard.distance <- function(x,y){
x <- as.vector(x)
y <- as.vector(y)
D <- abs(mean(x)-mean(y))/(sd(c(scale(x,center=TRUE,scale=FALSE),c(scale(y,center=TRUE,scale=FALSE)))))
return(D)
}

D.tot <- standard.distance(Y1%*%w,Y2%*%w)
I <- diag(1,p,p)
a0 <- as.vector(rep(1,p)/p)
M.k <- matrix(a0,ncol=1)
k.max <- kmax
best.ratio <- matrix(rep(0,2*k.max),ncol=2)
for (k in c(1:k.max)){
P.k <- I-M.k%*%solve(t(M.k)%*%M.k)%*%t(M.k)
Sigma.k <- P.k%*%Sigma%*%P.k
Sigma.k <- 0.5*(Sigma.k+t(Sigma.k))
ev <- eigen(Sigma.k)$values[1:(p-k)]
V <- eigen(Sigma.k)$vectors
Lambda.plus <- diag(c(1/ev,rep(0,k))) 
Sigma.plus <- V%*%Lambda.plus%*%t(V)
w.k <- Sigma.plus%*%P.k%*%m12
w.k <- as.matrix(w.k,ncol=1)
if (k==rd){ 
  D.bij <- standard.distance(Y1%*%w.k,Y2%*%w.k)
  D.shape <- standard.distance(Y1%*%w.k,Y2%*%w.k)/D.tot
  D.size <- standard.distance(Y1%*%a0,Y2%*%a0)/D.tot
  delta <- D.size/(D.size+D.shape)
}
c.max <- 0
for (i in c(1:(p-1))){  
  for (j in c((1+i):p)){
    b.ij <- rep(0,p)
    b.ij[i] <- 1
    b.ij[j] <- -1
    b.ij <- as.matrix(b.ij,ncol=1)  
    c.ij <- (t(b.ij)%*%Sigma%*%w.k)^2/(t(b.ij)%*%Sigma%*%b.ij)
    if (c.ij>c.max){
      best.b.ij <- b.ij 
      c.max <- c.ij
      best.ratio[k,1] <- i
      best.ratio[k,2] <- j 
    }    
}}
print(paste("Ratio no. ", as.character(k), ": ", trait.names[best.ratio[k,1]], "/", trait.names[best.ratio[k,2]], sep=""))
M.k <- cbind(M.k,Sigma%*%best.b.ij)
}
ratio1 <- x[,best.ratio[1,1]]/x[,best.ratio[1,2]]
ratio2 <- x[,best.ratio[2,1]]/x[,best.ratio[2,2]]
ratio3 <- x[,best.ratio[3,1]]/x[,best.ratio[3,2]]
ratio4 <- x[,best.ratio[4,1]]/x[,best.ratio[4,2]]

D.bij <- paste("D.bij of ratio no. ", as.character(rd), ": ", trait.names[best.ratio[rd,1]], " : ", trait.names[best.ratio[rd,2]], " = ", as.character(round(D.bij, roundD)), sep="")
D.shape <- paste("D.shape of ratio no. ", as.character(rd), ": ", trait.names[best.ratio[rd,1]], " : ", trait.names[best.ratio[rd,2]], " = ", as.character(round(D.shape, roundD)), sep="")
D.size <- paste("D.size of ratio no. ", as.character(rd), ": ", trait.names[best.ratio[rd,1]], " : ", trait.names[best.ratio[rd,2]], " = ", as.character(round(D.size, roundD)), sep="")
delta <- paste("delta of ratio no. ", as.character(rd), ": ", trait.names[best.ratio[rd,1]], " : ", trait.names[best.ratio[rd,2]], " = ", as.character(round(delta, roundD)), sep="")
list(ratio1=ratio1, ratio2=ratio2, ratio3=ratio3, ratio4=ratio4,  D.bij=D.bij, D.shape=D.shape, D.size=D.size, delta=delta)
}









###################################################
## weighting variables according to U/geometric mean(U)
###################################################

Xweigh <- function(X) {  
	X <- data.frame(X)  # transforming matrices, arrays etc. to a dataframe
	Yt <- data.frame(t(X))  # transposing dataframe
	x.gm <- exp(apply(log(X), 2, mean))  # geometric mean
	Y <- data.frame(t(Yt/x.gm)) # dividing transposed U by geometric mean
	Y
	}












###################################################
## creating single scatterplots with specimen label
###################################################

plotsingle <- function(x, y, label, color=1, cexlab=1, ...) {
	plot(x, y, col=color, pch=18, cex=cexlab,  ...)
	text(x, y, labels=label, pos=3, offset=0.2, srt=0, col="black", cex=0.8, ...)
	}
	








###################################################
## creating multiple scatterplots with specimen label
###################################################

plotmult <- function(x, y, z, ...) {
	p <- colnames(x)	
	for (i in 1:(ncol(x)-1)) for (j in (i+1):ncol(x)) {
		pi <- p[i]
		pj <- p[j]
	plot(x[,i], x[,j], col=y, pch=18, xlab=pi, ylab=pj)
	text(x[,i], x[,j], labels=z, pos=3, offset=0, srt=0, col="black", cex=1.2, ...)
	}
	}








###################################################
## creating multiple boxplots from a dataframe with measurements
###################################################

boxmult <- function(x, y) {
	p <- colnames(x)	
	for (i in 1:ncol(x)) boxplot(x[,i]~y, col = "lightgray", main=p[i])
	
	}


###################################################
## creating multiple boxplots from a dataframe with measurements
###################################################

ratioboxplot <- function(x, y) {
	p <- colnames(x)	
	for (i in 1:(ncol(x)-1)) for(j in (i+1):ncol(x)){
		pi <- p[i]
		pj <- p[j]
		pij <- paste(pi,pj, sep=" / ")
	
	boxplot(x[,i]/x[,j]~y, col = "lightgray", main=pij)
	}
	}

	

###################################################
## creating multiple scatterplots from a dataframe with raw data
###################################################

# x a factor variable, like species
# y a data frame or matrix with numeric variables
ratioplot <- function(x,y, ...) {
	p <- colnames(y)
	for (i in 1:(ncol(y)-1)) for(j in (i+1):ncol(y)){
		pi <- p[i]
		pj <- p[j]
		pij <- paste(pi, pj, sep=" / ")
		
	plot(x, y[,i]/y[,j], main=pij, ...)
	}
	}
	


###################################################
## creating multiple scatterplots from a dataframe of log-transformed raw data
###################################################
ratioplot.log <- function(x,y, ...) {
	p <- colnames(y)
	for (i in 1:(ncol(y)-1)) for(j in (i+1):ncol(y)){
		pi <- p[i]
		pj <- p[j]
		pij <- paste(pi, pj, sep=" / ")
		ratio <- log(y[,i]/y[,j])			
	plot(x, ratio, main=pij, ...)
	abline(lm(ratio~x))
	}
	}
	




###################################################
## creating multiple boxplots from a dataframe with measurements
###################################################
ratioplot2 <- function(x,y, asp=NA, xlab="", ylab="") {
	p <- colnames(y)
	for (i in 1:(ncol(y))){
		pi <- p[i]
		pij <- paste(pi)
		
	plot(x, y[,i], asp=asp, xlab=xlab, ylab=ylab, col = "red", cex=0.5, main=pij)
	}
	}
	





###################################################
## calculating ratios of variables in a dataframe and matrix
###################################################

ratios <- function(x) { 
	p <- colnames(x)
	k <- 0
	l <- 0
	A <- matrix(NA,nrow=nrow(x), ncol=ncol(x)*(ncol(x)-1)/2) 
	cn <- matrix(NA,nrow=1, ncol(x)*(ncol(x)-1)/2) 
	for (i in 1:(ncol(x)-1)) for(j in (i+1):ncol(x)){
	k <- k+1
	A[,k] <- x[,i]/x[,j]
	}
	for (i in 1:(ncol(x)-1)) for(j in (i+1):ncol(x)){
		l <- l+1
		pi <- p[i]
		pj <- p[j]
		pij <- paste(pi, pj, sep=".")
		cn[l] <- pij
	}
	colnames(A) <- cn
	A <- as.data.frame(A)
	A
}








###################################################
## calculating ratios of variables in a dataframe and matrix
###################################################

logratios <- function(x) { 
	p <- colnames(x)
	k <- 0
	l <- 0
	A <- matrix(NA,nrow=nrow(x), ncol=ncol(x)*(ncol(x)-1)/2) 
	cn <- matrix(NA,nrow=1, ncol(x)*(ncol(x)-1)/2) 
	for (i in 1:(ncol(x)-1)) for(j in (i+1):ncol(x)){
	k <- k+1
	A[,k] <- log(x[,i]/x[,j])
	}
	for (i in 1:(ncol(x)-1)) for(j in (i+1):ncol(x)){
		l <- l+1
		pi <- p[i]
		pj <- p[j]
		pij <- paste(pi, pj, sep=".")
		cn[l] <- pij
	}
	colnames(A) <- cn
	A <- as.data.frame(A)
	A
}








##########################################
#####  random numbers
##########################################
rn <- function(n, v) {
	i <- 0
	A <- matrix(NA, nrow=n, ncol=v)
	for (i in 1:v) A[,i] <- unique(rbinom(n*10, 10000000, 0.5))[1:n]
	A
}



#### geometric mean
g.mean <- function(x) exp(mean(log(x)))

#### standard deviation based on geometric mean
sd.gm <- function(x) sqrt(1/(length(x)-1)*sum((x-exp(mean(log(x))))^2))


# geometric mean regression (also reduced major axis)
gmeanreg <- function(y,x) {
	CL95intercept <- (intercept <- mean(y) - (slope <- sign(cor(x,
	y)) * sqrt(var(y)/var(x))) * mean(x)) + c(1, -1) * qt(0.025,
	df <- (n1 <- (n <- length(x)) - 1) - 1) * (SEintercept <- sqrt((MSE <- (var(y) -
	cov(x, y)^2/var(x)) * n1/df) * (1/n + mean(x)^2/var(x)/n1)))
	CL95slope <- slope + c(1, -1) * qt(0.025, df) * (SEslope <- sqrt(MSE/var(x)/n1))
	result1 <- c(slope, SEslope, CL95slope)
	result2 <- c(intercept, SEintercept, CL95intercept)
	names(result1) <- c("slope x", "SE[slope]", "95% LCL", "95% UCL")
	names(result2) <- c("intercept", "SE[int.]", "95% LCL", "95% UCL")
	list(RMAslope = result1, RMAintercept = result2)
	}


## zu überarbeiten
#plotmst erstellt minimal spannenden Baum fuer die Datenmatrix dat und zeichnet ihn in das Streudiagramm der Scores bezueglich der ersten beiden Hauptkomponenten
# in e steht das Ergebnis der Hauptkomponentenanalyse
# Dies kann man zum Beispiel durch e<-princomp(dat) erhalten
# zusammengestellt nach Handl (2002:138ff)
plotmst <- function(x){
   n<-dim(chal[,c(9:26)])[1]
   m<-mst(dist(chal[,c(9:26)]))
   v<-(1:n)[m[1,]==1]
   welche<-cbind(rep(1,length(v)),v)
   for(i in 2:(n-1)) {v<-(1:n)[m[i,]==1];
   welche<-rbind(welche,cbind(rep(i,length(v)),v))}
   x<-chal.1$PC1
   y<-chal.1$PC2
   quartz(width=12, height=12)
   plot(x,y,xlab="1. principal component",ylab="2. principal component",main="minimal spanning tree for PCA, NA=omit, without gst.l, gst.b")
   text(x,y,dimnames(chal[,c(9:26)])[[1]],pos=1)
   segments(x[welche[,1]],y[welche[,1]],x[welche[,2]],y[welche[,2]])
  }




###################################################
## bubble plots after Crawley, MJ: The R book (2007, p. 179-180)
###################################################
bubble.plot <- function(xv,yv,rv, bs = 0.1){
	r <- rv/max(rv)
	yscale <- max(yv) - min(yv)
	xscale <- max(xv) - min(xv)
	
	plot(xv,yv,type="n", xlab=deparse(substitute(xv)),
							ylab=deparse(substitute(yv)),
							main=deparse(substitute(rv)))
	
	for (i in 1:length(xv)) bubble(xv[i], yv[i],r[i],bs,xscale,yscale)
	}
	
bubble <- function(x,y,r,bubble.size,xscale,yscale) {
	theta <- seq(0,2*pi, pi/200)
	yv <- r*sin(theta)*bubble.size*yscale
	xv <- r*cos(theta)*bubble.size*xscale
	lines(x+xv, y+yv)	
	}	
	






